# Generated by Django 2.2.10 on 2020-03-05 13:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0004_auto_20200305_1848'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pagewidget',
            name='slot',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='django_power_cms.TemplateSlot', verbose_name='Slot'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-09 09:35

import ckeditor.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0010_auto_20200308_2206'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='content',
            field=ckeditor.fields.RichTextField(blank=True, null=True, verbose_name='Content'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-08 13:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0008_auto_20200308_2150'),
    ]

    operations = [
        migrations.AddField(
            model_name='articledetailwidget',
            name='show_description',
            field=models.BooleanField(default=False, verbose_name='Show Description'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-08 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0009_articledetailwidget_show_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='author',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Author'),
        ),
        migrations.AddField(
            model_name='articledetailwidget',
            name='show_author',
            field=models.BooleanField(default=False, verbose_name='Show Author'),
        ),
        migrations.AlterField(
            model_name='articledetailwidget',
            name='show_published_time',
            field=models.BooleanField(default=True, verbose_name='Show Published Time'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-07 13:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0005_auto_20200305_2136'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='code',
            field=models.CharField(max_length=64, verbose_name='Code'),
        ),
        migrations.AlterUniqueTogether(
            name='page',
            unique_together={('site', 'code')},
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-05 07:59

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlelistwidget',
            name='article_list_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='django_power_cms.Page', verbose_name='Article List Page'),
        ),
        migrations.AddField(
            model_name='articlelistwidget',
            name='article_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='django_power_cms.Page', verbose_name='Article Page'),
        ),
    ]



from django.shortcuts import render
from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.shortcuts import get_object_or_404
from .models import Page
from .models import Site
from django.contrib.auth.models import User

def site(request, site_code):
    site = get_object_or_404(Site, published=True, code=site_code)
    if site.index_page_code:
        url = reverse("django-power-cms.page", kwargs={"site_code": site.code, "page_code": site.index_page_code})
        return HttpResponseRedirect(url)
    else:
        raise Http404()

def page(request, site_code, page_code):
    page = get_object_or_404(Page,site__code=site_code, code=page_code)
    if not page.is_published:
        raise Http404()
    context = {
        "request": request,
        "site_code": site_code,
        "page_code": page_code,
        "site": page.site,
        "page": page,
        "title": page.site.name + " | " + page.name
    }
    html = page.render(request, context)
    return HttpResponse(html)




# Generated by Django 2.2.10 on 2020-03-05 10:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0003_template_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='theme',
            name='app_label',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='App Label'),
        ),
        migrations.AddField(
            model_name='theme',
            name='app_verbose_name',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='App Verbose Name'),
        ),
        migrations.AddField(
            model_name='theme',
            name='code',
            field=models.CharField(default='', max_length=32, unique=True, verbose_name='Code'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='theme',
            name='preview_image',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Preview Image'),
        ),
    ]



import djclick as click
import yaml
import pprint
from magic_import import import_from_string
from django_power_cms.models import Theme


@click.group()
def register():
    """Django-Power-Site-Theme register service.
    """

@register.command()
def scan():
    """Scan and list all avaiable themes.
    """
    from django.apps import apps
    counter = 0
    for app_config in apps.get_app_configs():
        name = app_config.name + ".export_themes.themes"
        themes = import_from_string(name) or []
        for theme in themes:
            counter += 1
            app_label = theme.get("app_label", "")
            code = theme.get("code", "")
            name = theme.get("name", "")
            print("{0}\t{1}\t{2}\t{3}".format(counter, app_label, code, name))

@register.command()
@click.argument("code", nargs=1, required=True)
def register(code):
    """Register or update theme by theme code.
    """
    found = False
    from django.apps import apps
    for app_config in apps.get_app_configs():
        name = app_config.name + ".export_themes.themes"
        themes = import_from_string(name) or []
        for theme in themes:
            theme_code = theme.get("code", "")
            theme_name = theme.get("name", "")
            app_label = theme.get("app_label", "")
            if code == theme_code:
                found = True
                Theme.register(theme)
                print("Theme registered: {0} {1} {2}".format(app_label, theme_code, theme_name))
                break
    if not found:
        print("Theme with code: {0} NOT found.".format(code))


# Generated by Django 2.2.11 on 2020-03-19 04:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0013_auto_20200319_1039'),
    ]

    operations = [
        migrations.AddField(
            model_name='site',
            name='favicon',
            field=models.ImageField(blank=True, null=True, upload_to='site/favicons/', verbose_name='FavIcon'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-21 06:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0014_site_favicon'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='articles', to='django_power_cms.Site', verbose_name='Site'),
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-08 13:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0006_auto_20200307_2148'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='templateslot',
            options={'ordering': ['template', 'code'], 'verbose_name': 'Template Slots'},
        ),
        migrations.CreateModel(
            name='ArticleDetailWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('root', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_power_cms.Article', verbose_name='Root')),
            ],
            options={
                'verbose_name': 'Article Detail Widget',
                'verbose_name_plural': 'Article Detail Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
    ]



import json
import logging
from fastutils import listutils
from django.db import models
from django.template.loader import render_to_string
from django.apps import apps
from django.utils.html import mark_safe
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from mptt.models import MPTTModel, TreeForeignKey
from django_ckeditor_5.fields import CKEditor5Field
from ckeditor.fields import RichTextField
from django_msms_admin.models import DjangoMsmsModelAbstractBase


logger = logging.getLogger(__name__)


class Theme(models.Model):
    code = models.CharField(max_length=32, unique=True, verbose_name=_("Code"))
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    app_label= models.CharField(max_length=64, null=True, blank=True, verbose_name=_("App Label"))
    app_verbose_name = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("App Verbose Name"))
    description = models.TextField(null=True, blank=True, verbose_name=_("Description"))
    preview_image = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Preview Image"))
    is_default = models.BooleanField(default=False, verbose_name=_("Is default theme"))

    class Meta:
        verbose_name = _("Theme")
        verbose_name_plural = _("Themes")

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        result = super().save(*args, **kwargs)
        if self.is_default:
            for theme in Theme.objects.filter(is_default=True).exclude(pk=self.pk):
                theme.is_default = False
        return result

    @property
    def css(self):
        return [x.css for x in self.css_files.all()]

    @property
    def js(self):
        return [x.js for x in self.js_files.all()]

    @classmethod
    def get_default_theme(cls):
        return cls.objects.filter(is_default=True).all()[0]

    @classmethod
    def register(cls, config):
        try:
            cls._register(config)
        except Exception as error:
            logger.exception("Register Theme failed: {0}".format(str(config)))

    @classmethod
    def _register(cls, config):
        code = config["code"]
        try:
            instance = Theme.objects.get(code=code)
        except Theme.DoesNotExist:
            instance = Theme()
            instance.code = code
        instance.name = config["name"]
        instance.app_label = config["app_label"]
        instance.app_verbose_name = config["app_verbose_name"]
        instance.description = config["description"]
        instance.preview_image = config["preview_image"]
        instance.save()
        old_css = dict([(x.css, x) for x in instance.css_files.all()])
        old_js = dict([(x.js, x) for x in instance.js_files.all()])
        # css handler
        for css in config["css"]:
            if not css in old_css:
                css_instance = ThemeCss()
                css_instance.theme = instance
                css_instance.css = css
                css_instance.order = 0
                css_instance.save()
            else:
                del old_css[css]
        for css, css_instance in old_css.items():
            css_instance.delete()
        old_css = dict([(x.css, x) for x in instance.css_files.all()])
        index = 0
        for css in config["css"]:
            index += 1000
            if old_css[css].order != index:
                old_css[css].order = index
                old_css[css].save()
        # js handler
        for js in config["js"]:
            if not js in old_js:
                js_instance = ThemeCss()
                js_instance.theme = instance
                js_instance.js = js
                js_instance.order = 0
                js_instance.save()
            else:
                del old_js[js]
        for js, js_instance in old_js.items():
            js_instance.delete()
        old_js = dict([(x.js, x) for x in instance.js_files.all()])
        index = 0
        for js in config["js"]:
            index += 1000
            if old_js[js].order != index:
                old_js[js].order = index
                old_js[js].save()

class ThemeCss(models.Model):
    theme = models.ForeignKey(Theme, on_delete=models.CASCADE, related_name="css_files", verbose_name=_("Theme"))
    css = models.CharField(max_length=128, verbose_name=_("CSS File"))
    order = models.IntegerField(verbose_name=_("Order"))

    class Meta:
        verbose_name = _("Theme CSS")
        verbose_name_plural = _("Theme CSSes")
        ordering = ["order"]

class ThemeJs(models.Model):
    theme = models.ForeignKey(Theme, on_delete=models.CASCADE, related_name="js_files", verbose_name=_("Theme"))
    js = models.CharField(max_length=128, verbose_name=_("JS File"))
    order = models.IntegerField(verbose_name=_("Order"))

    class Meta:
        verbose_name = _("Theme JS")
        verbose_name_plural = _("Theme JSes")
        ordering = ["order"]

class Template(models.Model):
    code = models.CharField(max_length=32, unique=True, verbose_name=_("Code"))
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    app_label= models.CharField(max_length=64, null=True, blank=True, verbose_name=_("App Label"))
    app_verbose_name = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("App Verbose Name"))
    description = models.TextField(null=True, blank=True, verbose_name=_("Description"))
    preview_image = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Preview Image"))
    template = models.CharField(max_length=128, verbose_name=_("Template File"))

    class Meta:
        verbose_name = _("Template")
        verbose_name_plural = _("Templates")

    def __str__(self):
        return self.name

    @classmethod
    def register(cls, config):
        try:
            cls._register(config)
        except Exception as error:
            logger.exception("Register Template failed: {0}".format(str(config)))

    @classmethod
    def _register(cls, config):
        code = config["code"]
        try:
            instance = Template.objects.get(code=code)
        except Template.DoesNotExist:
            instance = Template()
            instance.code = code
        instance.name = config["name"]
        instance.app_label = config["app_label"]
        instance.app_verbose_name = config["app_verbose_name"]
        instance.description = config["description"]
        instance.preview_image = config["preview_image"]
        instance.template = config["template"]
        instance.save()
        old_slots = dict([(x.code, x) for x in instance.slots.all()])
        for slot_config in config["slots"]:
            code = slot_config["code"]
            name = slot_config["name"]
            order = slot_config["order"]
            if not code in old_slots:
                slot_instance = TemplateSlot()
                slot_instance.template = instance
                slot_instance.code = code
                slot_instance.name = name
                slot_instance.order = order
                slot_instance.save()
            else:
                if old_slots[code].name != name:
                    old_slots[code].name = name
                    old_slots[code].save()
                del old_slots[code]
        for code, slot in old_slots.items():
            slot.delete()

class TemplateSlot(models.Model):
    template = models.ForeignKey(Template, on_delete=models.CASCADE, related_name="slots", verbose_name=_("Template"))
    code = models.CharField(max_length=32, verbose_name=_("Code"))
    name = models.CharField(max_length=32, verbose_name=_("Name"))
    order = models.IntegerField(null=True, blank=True, verbose_name=_("Order"))

    class Meta:
        verbose_name = _("Template Slot")
        verbose_name = _("Template Slots")
        ordering = ["template", "order"]
        unique_together = [
            ("template", "code"),
        ]

    def __str__(self):
        return "{0}-{1}".format(self.template.name, self.name)

class Site(models.Model):
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    code = models.CharField(max_length=64, unique=True, verbose_name=_("Code"))
    published = models.BooleanField(verbose_name=_("Published"))
    published_time = models.DateTimeField(null=True, blank=True, verbose_name=_("Published Time"))
    theme = models.ForeignKey(Theme, on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Theme"))
    index_page_code = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("Index Page Code"))
    favicon = models.ImageField(upload_to="site/favicons/", null=True, blank=True, verbose_name=_("FavIcon"))

    class Meta:
        verbose_name = _("Site")
        verbose_name_plural = _("Sites")


    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.published and (not self.published_time):
            self.published_time = timezone.now()
        if not self.published:
            self.published_time = None
        return super().save(*args, **kwargs)

    @property
    def is_published(self):
        return self.published

    def get_theme(self):
        if self.theme:
            return self.theme
        else:
            return Theme.get_default_theme()

    def get_absolute_url(self):
        return reverse("django-power-cms.site", kwargs={"site_code": self.code})

class Page(MPTTModel):
    site = models.ForeignKey(Site, on_delete=models.CASCADE, related_name="pages", verbose_name=_("Site"))
    parent = TreeForeignKey("self", on_delete=models.SET_NULL, null=True, blank=True, related_name="children", verbose_name=_("Parent Page"))
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    code = models.CharField(max_length=64, verbose_name=("Code"))
    order = models.IntegerField(null=True, blank=True, verbose_name=_("Order"))
    published = models.NullBooleanField(verbose_name=_("Published"))
    published_time = models.DateTimeField(null=True, blank=True, verbose_name=_("Published Time"))
    template = models.ForeignKey(Template, on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Template"))
    theme = models.ForeignKey(Theme, on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Theme"))

    class Meta:
        verbose_name = _("Page")
        verbose_name_plural = _("Pages")
        unique_together = [
            ("site", "code"),
        ]
    
    class MPTTMeta:
        order_insertion_by = ['order']

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.published and (not self.published_time):
            self.published_time = timezone.now()
        if not self.published:
            self.published_time = None
        return super().save(*args, **kwargs)

    @property
    def is_published(self):
        if self.published == True:
            return True
        if self.published == False:
            return False
        if self.parent:
            return self.parent.is_published
        else:
            return self.site.is_published

    def get_absolute_url(self):
        return reverse("django-power-cms.page", kwargs={"site_code": self.site.code, "page_code": self.code})

    def get_theme(self):
        if self.theme:
            return self.theme
        elif self.parent:
            return self.parent.get_theme()
        else:
            return self.site.get_theme()

    def render(self, request, global_context):
        css_links = []
        js_links = []
        # append theme resources
        theme = self.get_theme()
        for css in theme.css:
            listutils.append_new(css_links, css)
        for js in theme.js:
            listutils.append_new(js_links, js)
        # render widgets
        # and append widget resources
        widgets = {}
        for binding in self.widget_bindings.all():
            if not binding.slot: # if PageWidget NOT provide slot, just ignore it
                continue
            if not binding.slot.code in widgets:
                widgets[binding.slot.code] = []
            widgets[binding.slot.code].append(binding.widget.render(request, global_context))
            for css in binding.widget.get_real_model().css:
                listutils.append_new(css_links, css)
            for js in binding.widget.get_real_model().js:
                listutils.append_new(js_links, js)
        # make context
        context = {}
        context.update(global_context or {})
        context["widgets"] = widgets
        context["css_links"] = css_links
        context["js_links"] = js_links
        # render to string
        return render_to_string(self.template.template, context, request=request)

class PageWidget(models.Model):
    page = models.ForeignKey(Page, on_delete=models.CASCADE, related_name="widget_bindings", verbose_name=_("Page"))
    slot = models.ForeignKey(TemplateSlot, on_delete=models.SET_NULL, null=True, blank=True, related_name="+", verbose_name=_("Slot"))
    order = models.IntegerField(default=0, verbose_name=_("Order"))
    widget = models.ForeignKey("Widget", on_delete=models.CASCADE, related_name="+", verbose_name=_("Widget"))

    class Meta:
        verbose_name = _("Page Widget Binding")
        verbose_name_plural = _("Page Widget Bindings")
        ordering = ["slot", "order"]

    def __str__(self):
        return str(self.pk)

class Widget(DjangoMsmsModelAbstractBase, models.Model):
    name = models.CharField(max_length=64, verbose_name=_("Name"))
    # widget box settings
    title = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Title"))
    widget_with_border = models.BooleanField(default=False, verbose_name=_("Widget With border"))
    widget_body_padding = models.CharField(max_length=16, default="10px", null=True, blank=True, verbose_name=_("Widget Body Pading"))
    widget_style = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Style"))
    widget_header_style = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Header Style"))
    widget_body_style = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Body Style"))
    widget_footer_style = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Footer Style"))
    widget_class = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Class"))
    widget_header_class = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Header Class"))
    widget_body_class = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Body Class"))
    widget_footer_class = models.CharField(max_length=256, null=True, blank=True, verbose_name=_("Widget Footer Class"))

    css = []
    js = []

    class Meta:
        verbose_name = _("Widget")
        verbose_name_plural = _("Widgets")

    def __str__(self):
        return self.name

    def render(self, request, global_context):
        context = {}
        context.update(global_context or {})
        header, footer = self.widget_header_and_footer_render(request, global_context)
        real_widget_model = self.get_real_model()
        if real_widget_model:
            body = real_widget_model.objects.get(pk=self.pk).render(request, global_context)
            context.update({
                "pk": self.pk,
                "widget_with_border": self.widget_with_border,
                "widget_body_padding": self.widget_body_padding or "",
                "widget_style": self.widget_style or "",
                "widget_header_style": self.widget_header_style or "",
                "widget_body_style": self.widget_body_style or "",
                "widget_footer_style": self.widget_footer_style or "",
                "widget_class": self.widget_class or "",
                "widget_header_class": self.widget_header_class or "",
                "widget_body_class": self.widget_body_class or "",
                "widget_footer_class": self.widget_footer_class or "",
                "header": header.strip(),
                "body": body.strip(),
                "footer": footer.strip(),
            })
            return render_to_string("django-power-cms/widgets/power-widget.html", context)
        else:
            return ""

    def widget_header_and_footer_render(self, request, global_context):
        top_left_links = []
        top_right_links = []
        bottom_left_links = []
        bottom_right_links = []
        links = self.links.all()
        for link in links:
            if link.role == WidgetLink.TOP_LEFT:
                top_left_links.append(link)
            elif link.role == WidgetLink.TOP_RIGHT:
                top_right_links.append(link)
            elif link.role == WidgetLink.BOTTOM_LEFT:
                bottom_left_links.append(link)
            elif link.role == WidgetLink.BOTTOM_RIGHT:
                bottom_right_links.append(link)
        header_context = {}
        footer_context = {}
        header_context.update(global_context or {})
        footer_context.update(global_context or {})
        header_context.update({
            "title": self.title,
            "top_left_links": top_left_links,
            "top_right_links": top_right_links,
        })
        footer_context.update({
            "bottom_left_links": bottom_left_links,
            "bottom_right_links": bottom_right_links,
        })
        header = render_to_string("django-power-cms/widgets/power-widget-header.html", header_context)
        footer = render_to_string("django-power-cms/widgets/power-widget-footer.html", footer_context)
        return header, footer

class WidgetLink(models.Model):
    TOP_LEFT = 1
    TOP_RIGHT = 2
    BOTTOM_LEFT = 3
    BOTTOM_RIGHT = 4
    ROLES = [
        (TOP_LEFT, _("Top Left")),
        (TOP_RIGHT, _("Top Right")),
        (BOTTOM_LEFT, _("Bottom Left")),
        (BOTTOM_RIGHT, _("Bottom Right")),
    ]

    Widget = models.ForeignKey(Widget, on_delete=models.CASCADE, related_name="links", verbose_name=_("Widget"))
    role = models.IntegerField(choices=ROLES, verbose_name=_("Widget Link Role"))
    title = models.CharField(max_length=32, verbose_name=_("Title"))
    url = models.CharField(max_length=128, verbose_name=_("Url"))
    icon = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("Icon Class"))
    target = models.CharField(max_length=16, null=True, blank=True, verbose_name=_("Link Target"))

    class Meta:
        verbose_name = _("Widget Link")
        verbose_name_plural = _("Widget Links")

class StaticHtmlWidget(Widget):
    html = models.TextField(null=True, blank=True, verbose_name=_("HTML Code"))

    class Meta:
        verbose_name = _("Static HTML Widget")
        verbose_name_plural = _("Static HTML Widgets")

    def render(self, request, global_context):
        return self.html

class CarouselWidget(Widget):
    width = models.IntegerField(null=True, blank=True, verbose_name=_("Width"))
    height = models.IntegerField(null=True, blank=True, verbose_name=_("Height"))

    css = [
        "bootstrap/css/bootstrap.min.css",
        "django-power-cms/widgets/carousel/css/carousel.css",
    ]
    js = [
        "jquery3/jquery.js",
        "bootstrap/js/bootstrap.min.js",
    ]

    class Meta:
        verbose_name = _("Carousel Widget")
        verbose_name_plural = _("Carousel Widgets")

    def render(self, request, global_context):
        return render_to_string("django-power-cms/widgets/carousel.html", {
            "pk": self.pk,
            "images": self.images.all(),
            "width": self.width,
            "height": self.height,
        })

class CarouselWidgetImage(models.Model):
    carousel = models.ForeignKey(CarouselWidget, on_delete=models.CASCADE, related_name="images", verbose_name=_("Carousel"))
    image = models.ImageField(upload_to="carousel-images", verbose_name=_("Image"))
    url = models.CharField(max_length=512, null=True, blank=True, verbose_name=_("URL"))
    target = models.CharField(max_length=32, null=True, blank=True, verbose_name=_("URL Target"))
    order = models.IntegerField(default=0, verbose_name=_("Order"))

    class Meta:
        verbose_name = _("Carousel Widget Image")
        verbose_name_plural =  _("Carousel Widget Images")
        ordering = ["order"]

    def __str__(self):
        return str(self.pk)

class StaticListWidget(Widget):
    max_display_count = models.IntegerField(null=True, blank=True, verbose_name=_("Max Display Count"))
    empty_display_message = models.TextField(null=True, blank=True, verbose_name=_("Empty Display Message"))

    class Meta:
        verbose_name = _("Static List Widget")
        verbose_name_plural = _("Static List Widgets")

    def render(self, request, global_context):
        if self.max_display_count:
            lists = self.lists.filter(published=True).order_by("order").all()[:self.max_display_count]
        else:
            lists = self.lists.filter(published=True).order_by("order").all()
        return render_to_string("django-power-cms/widgets/static-list.html", {
            "lists": lists,
            "lists_length": len(lists),
            "empty_display_message": self.empty_display_message,
        })

class StaticListItem(models.Model):
    thelist = models.ForeignKey(StaticListWidget, on_delete=models.CASCADE, related_name="lists", verbose_name=_("The Static List Widget"))
    title = models.CharField(max_length=128, verbose_name=_("Title"))
    url = models.CharField(max_length=512, null=True, blank=True, verbose_name=_("URL"))
    target = models.CharField(max_length=32, null=True, blank=True, verbose_name=_("URL Target"))
    order = models.IntegerField(default=0, verbose_name=_("Order"))
    label = models.CharField(max_length=128, verbose_name=_("Label"))
    label_class = models.CharField(max_length=32, verbose_name=_("Label Class"))

    class Meta:
        ordering = ["order"]

    def __str__(self):
        return self.title

class TopbarWidget(Widget):
    brand_link = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Brand Link"))
    brand_link_target = models.CharField(max_length=32, null=True, blank=True, verbose_name=_("Brand Link Target"))
    welcome_message_for_login_user = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Welcome Message For Login User"))
    welcome_message_for_anonymous_user = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Welcome Message For Anonymous User"))
    fix_position = models.BooleanField(default=False, verbose_name=_("Fix Position"))
    show_login_or_logout_link = models.BooleanField(default=False, verbose_name=_("Show Login Or Logout Link"))
    login_link = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Login Link"))
    logout_link = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Logout Link"))
    show_change_password_link = models.BooleanField(default=False, verbose_name=_("Show Change Password Link"))
    change_password_link = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("Change Password Link"))

    class Meta:
        verbose_name = _("Topbar Widget")
        verbose_name_plural = _("Topbar Widgets")

    def render(self, request, global_context):
        context = {}
        context.update(global_context or {})
        brands = self.brands.all()
        context.update({
            "brand_link": self.brand_link,
            "brand_link_target": self.brand_link_target,
            "brands": brands,
            "welcome_message": self.get_welcome_message(request),
            "welcome_links": self.get_welcome_links(request),
        })
        return render_to_string("django-power-cms/widgets/topbar.html", context)

    def get_welcome_message(self, request):
        if request.user and request.user.pk:
            if not self.welcome_message_for_login_user:
                return ""
            else:
                return self.welcome_message_for_login_user.format(fullname=request.user.get_full_name())
        else:
            if not self.welcome_message_for_anonymous_user:
                return ""
            else:
                return self.welcome_message_for_anonymous_user
    
    def get_welcome_links(self, request):
        links = []
        change_password_link = self.get_change_password_link(request)
        if change_password_link:
            links.append(change_password_link)
        login_or_logout_link = self.get_login_or_logout_link(request)
        if login_or_logout_link:
            links.append(login_or_logout_link)
        links.reverse()
        return links

    def get_login_or_logout_link(self, request):
        if not self.show_login_or_logout_link:
            return None
        if request.user and request.user.pk:
            return {
                "class": "logout-link",
                "title": _("Logout"),
                "url": self.logout_link,
            }
        else:
            return {
                "class": "login-link",
                "title": _("Login"),
                "url": self.login_link,
            }

    def get_change_password_link(self, request):
        if not (self.show_change_password_link and request.user and request.user.pk):
            return None
        else:
            return {
                "class": "change-password-link",
                "title": _("Change password"),
                "url": self.change_password_link,
            }

class TopbarBrand(models.Model):
    topbar = models.ForeignKey(TopbarWidget, on_delete=models.CASCADE, related_name="brands", verbose_name=_("Topbar"))
    image = models.ImageField(upload_to="brand-images", null=True, blank=True, verbose_name=_("Image"))
    title = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("Title"))
    url_raw = models.CharField(max_length=128, null=True, blank=True, verbose_name=_("URL"))
    order = models.IntegerField(default=0, verbose_name=_("Order"))

    class Meta:
        ordering = ["order"]
        verbose_name = _("Topbar Brand")
        verbose_name_plural = _("Topbar Brands")

    def url(self):
        if not self.url:
            return "#"
        if self.url_raw.startswith("site:"):
            site_code = self.url_raw[5:]
            return reverse("django-power-cms.site", kwargs={"site_code": site_code})
        if self.url_raw.startswith("page:"):
            _, site_code, page_code = self.url_raw.split(":")
            return reverse("django-power-cms.page", kwargs={"site_code": site_code, "page_code": page_code})
        return self.url_raw
    url.short_description = _("URL")


class Article(MPTTModel):
    TOP = _("Set Top")
    IMPORTANT = _("Important")
    GOOD_NEWS = _("Good News")
    WARNING = _("Warning")
    LABEL_CHOICES = [
        (TOP, TOP),
        (IMPORTANT, IMPORTANT),
        (GOOD_NEWS, GOOD_NEWS),
        (WARNING, WARNING),
    ]
    LABEL_CLASS_MAP = {
        str(TOP): "label label-primary",
        str(IMPORTANT): "label label-primary",
        str(GOOD_NEWS): "label label-success",
        str(WARNING): "label label-warning",
    }
    HIGH = 0
    NORAML = 99
    site = models.ForeignKey(Site, on_delete=models.CASCADE, null=True, blank=True, related_name="articles", verbose_name=_("Site"))
    parent = TreeForeignKey("self", on_delete=models.SET_NULL, null=True, blank=True, related_name="children", verbose_name=_("Article Parent"))
    title = models.CharField(max_length=64, verbose_name=_("Title"))
    author = models.CharField(max_length=64, null=True, blank=True, verbose_name=_("Author"))
    description = models.TextField(null=True, blank=True, verbose_name=_("Description"))
    content = RichTextField(null=True, blank=True, verbose_name=_("Content"))
    published = models.BooleanField(verbose_name=_("Published"))
    published_time = models.DateTimeField(null=True, blank=True, verbose_name=_("Published Time"))
    preview_image = models.ImageField(upload_to="article-preview-images", null=True, blank=True, verbose_name=_("Preview Image"))
    show_preview_image = models.BooleanField(default=False, verbose_name=_("Show Preview Image"))
    order = models.IntegerField(null=True, blank=True, verbose_name=_("Order"))
    priority = models.IntegerField(default=99, verbose_name=_("Priority"))
    label = models.CharField(max_length=16, choices=LABEL_CHOICES, null=True, blank=True, verbose_name=_("Label"))

    class Meta:
        verbose_name = _("Article")
        verbose_name_plural = _("Articles")

    class MPTTMeta:
        order_insertion_by = ['order']

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        if self.label:
            self.priority = self.HIGH
        else:
            self.priority = self.NORAML
        return super().save(*args, **kwargs)

    @property
    def label_class(self):
        return self.LABEL_CLASS_MAP.get(self.label, "")

class ArticleContentImage(models.Model):
    article = models.ForeignKey(Article, on_delete=models.CASCADE, related_name="images", verbose_name=_("Article"))
    image = models.ImageField(upload_to="article-content-images", verbose_name=_("Image"))

    class Meta:
        verbose_name = _("Article Content Image")
        verbose_name_plural = _("Article Content Images")

    def __str__(self):
        return str(self.pk)

    def image_link(self):
        if not self.image:
            return ""
        else:
            return self.image.url
    image_link.short_description = _("Image Link")

class ArticleListWidget(Widget):
    root = models.ForeignKey("Article", on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Root"))
    article_list_page = models.ForeignKey(Page, on_delete=models.SET_NULL, related_name="+", null=True, blank=True, verbose_name=_("Article List Page"))
    article_page = models.ForeignKey(Page, on_delete=models.SET_NULL, related_name="+", null=True, blank=True, verbose_name=_("Article Page"))
    max_display_count = models.IntegerField(default=10, verbose_name=_("Max Display Count"))
    empty_display_message = models.TextField(null=True, blank=True, verbose_name=_("Empty Display Message"))

    class Meta:
        verbose_name = _("Article List Widget")
        verbose_name_plural = _("Article List Widgets")

    def render(self, request, global_context):
        lists = self.root.children.filter(published=True).order_by("priority").all()[:self.max_display_count]
        return render_to_string("django-power-cms/widgets/article-list.html", {
            "lists": lists,
            "lists_length": len(lists),
            "empty_display_message": self.empty_display_message,
            "article_page": self.article_page,
            "article_list_page": self.article_list_page,
        })

class ArticleDetailWidget(Widget):
    root = models.ForeignKey("Article", on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Root"))
    show_description = models.BooleanField(default=False, verbose_name=_("Show Description"))
    show_published_time = models.BooleanField(default=True, verbose_name=_("Show Published Time"))
    show_author = models.BooleanField(default=False, verbose_name=_("Show Author"))
    show_prev_and_next_links = models.BooleanField(default=False, verbose_name=_("Show Prev&Next Links"))

    class Meta:
        verbose_name = _("Article Detail Widget")
        verbose_name_plural = _("Article Detail Widgets")

    def render(self, request, global_context):
        article_id= int(request.GET.get("article_id"))
        article = Article.objects.get(pk=article_id)
        if self.show_prev_and_next_links:
            prev = article.get_previous_sibling()
            next = article.get_next_sibling()
        else:
            prev = None
            next = None
        return render_to_string("django-power-cms/widgets/article-detail.html", {
            "article_id": article_id,
            "article": article,
            "show_description": self.show_description,
            "show_published_time": self.show_published_time,
            "show_prev_and_next_links": self.show_prev_and_next_links,
            "prev": prev,
            "next": next,
        })



from django.template import Library
from ..models import Site
from ..models import Page

register = Library()


@register.filter
def smarturl(url):
    if not url:
        return "#"
    elif url.startswith("site:"):
        _, site_code = url.split(":")
        try:
            return Site.objects.get(code=site_code).get_absolute_url()
        except Site.DoesNotExist:
            return "#"
    elif url.startswith("page:"):
        _, site_code, page_code = url.split(":")
        try:
            return Page.objects.get(site__code=site_code, code=page_code).get_absolute_url()
        except Page.DoesNotExist:
            return "#"
    else:
        return url



# Generated by Django 2.2.11 on 2020-03-19 02:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0012_auto_20200309_2121'),
    ]

    operations = [
        migrations.RenameField(
            model_name='topbarbrand',
            old_name='url',
            new_name='url_raw',
        ),
    ]



from django_power_cms.apps import DjangoPowerCmsConfig
from django.utils.translation import ugettext_lazy as _

themes = [{
    "code": "7ce32fbb3d4b41dfbe592ba28487a175",
    "name": _("django2-style"),
    "app_label": "django_power_cms",
    "app_verbose_name": DjangoPowerCmsConfig.verbose_name,
    "description": _("Django2 style theme Description."),
    "preview_image": "django-power-cms/theme-preview-image/django2-style.png",
    "css": [
        "django-power-cms/themes/django2-style/css/django2-style.css",
    ],
    "js": [
        
    ]
}]



from django.urls import path
from . import views


urlpatterns = [
    path('<site_code>/', views.site, name="django-power-cms.site"),
    path('<site_code>/<page_code>/', views.page, name="django-power-cms.page"),
    
 ]
 


# Generated by Django 2.2.11 on 2020-03-08 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0007_auto_20200308_2114'),
    ]

    operations = [
        migrations.AddField(
            model_name='articledetailwidget',
            name='show_prev_and_next_links',
            field=models.BooleanField(default=False, verbose_name='Show Prev&Next Links'),
        ),
        migrations.AddField(
            model_name='articledetailwidget',
            name='show_published_time',
            field=models.BooleanField(default=False, verbose_name='Show Published Time'),
        ),
    ]



from django.apps import AppConfig
from django_admin_app_sidebar.portlets import register_portlet
from django_admin_app_sidebar.portlets import SidebarNavigation
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse_lazy
from django.urls import reverse

class DjangoPowerCmsConfig(AppConfig):
    name = 'django_power_cms'
    verbose_name = _("Django Power CMS")

    def ready(self):
        self.register_app_navigation_portlet()
        self.model_list_reorder()

    def get_site_config_url(self, request, *args, **kwargs):
        from .admin import CURRENT_SIET_ID_SESSION_KEY
        try:
            site_id = request.session[CURRENT_SIET_ID_SESSION_KEY]
            return reverse("admin:{}_{}_change".format(self.name, "site"), kwargs={"object_id": int(site_id)})
        except:
            return """javascript:alert("请先选择站点！");"""

    def get_page_manager_url(self, request, *args, **kwargs):
        from .admin import CURRENT_SIET_ID_SESSION_KEY
        try:
            site_id = request.session[CURRENT_SIET_ID_SESSION_KEY]
            return reverse("admin:{}_{}_changelist".format(self.name, "page"))
        except:
            return """javascript:alert("请先选择站点！");"""

    def get_widget_manager_url(self, request, *args, **kwargs):
        from .admin import CURRENT_SIET_ID_SESSION_KEY
        try:
            site_id = request.session[CURRENT_SIET_ID_SESSION_KEY]
            return reverse("admin:{}_{}_changelist".format(self.name, "widget"))
        except:
            return """javascript:alert("请先选择站点！");"""

    def get_article_manager_url(self, request, *args, **kwargs):
        from .admin import CURRENT_SIET_ID_SESSION_KEY
        try:
            site_id = request.session[CURRENT_SIET_ID_SESSION_KEY]
            return reverse("admin:{}_{}_changelist".format(self.name, "article"))
        except:
            return """javascript:alert("请先选择站点！");"""

    def register_app_navigation_portlet(self):
        nav = SidebarNavigation(_("Site Manager"), [{
            "title": "站点选择",
            "url": reverse_lazy("admin:{0}_{1}_changelist".format(self.name, "site")),
            "icon": "fas fa-globe",
            "depth": 1,
        }, {
            "title": "站点设置",
            "url": self.get_site_config_url,
            "icon": "fas fa-cog",
            "depth": 1,
        }, "-", {
            "title": _("Page Manager"),
            "url": self.get_page_manager_url,
            "icon": "fas fa-object-group",
            "depth": 1,
        }, {
            "title": _("Widget Manager"),
            "url": self.get_widget_manager_url,
            "icon": "fas fa-th",
            "depth": 1,
        }, {
            "title": _("Article Manager"),
            "url": self.get_article_manager_url,
            "icon": "fas fa-newspaper",
            "depth": 1,
        }])
        register_portlet(nav, app_label=self.name, model_name="site")
        register_portlet(nav, app_label=self.name, model_name="page")
        register_portlet(nav, app_label=self.name, model_name="widget")
        register_portlet(nav, app_label=self.name, model_name="article")

    def model_list_reorder(self):
        from django.conf import settings
        if hasattr(settings, "ADMIN_REORDER"):
            for app_index in range(len(settings.ADMIN_REORDER)):
                app_setting = settings.ADMIN_REORDER[app_index]
                if isinstance(app_setting, str):
                    if app_setting != "django_power_cms":
                        continue
                    else:
                        app_setting = {"app": "django_power_cms"}
                        settings.ADMIN_REORDER[app_index] = app_setting
                else:
                    if app_setting["app"] != "django_power_cms":
                        continue
                app_setting["models"] =  [
                    {"model":  'django_power_cms.Site', "label": _("Site Manager")},
                    {"model": 'django_power_cms.Template', "label": _("Template Manager")},
                    {"model": 'django_power_cms.Theme', "label": _("Theme Manager")},
                ]



from django.contrib import admin
from django.forms import ModelForm
from django.urls import reverse
from django.contrib.admin.options import csrf_protect_m
from mptt.admin import DraggableMPTTAdmin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from django_readedit_switch_admin.admin import DjangoReadEditSwitchAdmin
from django_changelist_toolbar_admin.admin import DjangoChangelistToolbarAdminMixin
from django_visit_on_site_in_new_window.admin import DjangoVisitOnSiteInNewWindowAdmin
from django_cards_admin.admin import DjangoCardsAdminMixin
from django_msms_admin.admin import DjangoMsmsAdmin
from django_msms_admin.admin import DjangoSubclassAdmin
from django_tabbed_changeform_admin.admin import DjangoTabbedChangeformAdmin
from django_toggle_switch_widget.widgets import DjangoToggleSwitchWidget
from .models import Template
from .models import TemplateSlot
from .models import Site
from .models import Page
from .models import PageWidget
from .models import Widget
from .models import StaticHtmlWidget
from .models import CarouselWidget
from .models import CarouselWidgetImage
from .models import Theme
from .models import ThemeCss
from .models import ThemeJs
from .models import WidgetLink
from .models import StaticListWidget
from .models import StaticListItem
from .models import TopbarWidget
from .models import TopbarBrand
from .models import Article
from .models import ArticleContentImage
from .models import ArticleListWidget
from .models import ArticleDetailWidget

CURRENT_SIET_ID_SESSION_KEY = "django-power-cms-current-site-id"

class TemplateSlotInline(DjangoReadEditSwitchAdmin, admin.TabularInline):
    model = TemplateSlot
    extra = 0

class TemplateAdmin(DjangoReadEditSwitchAdmin, admin.ModelAdmin):
    list_display = ["name", "app_label", "template", "preview_image"]
    list_filter = ["app_label"]
    search_fields = ["name", "description", "template"]
    inlines = [
        TemplateSlotInline,
    ]


class ThemeCssInline(DjangoReadEditSwitchAdmin, admin.TabularInline):
    model = ThemeCss
    extra = 0

class ThemeJsInline(DjangoReadEditSwitchAdmin, admin.TabularInline):
    model = ThemeJs
    extra = 0

class ThemeAdmin(DjangoReadEditSwitchAdmin, admin.ModelAdmin):
    list_display = ["name", "description", "is_default"]
    list_filter = ["is_default"]
    search_fields = ["name", "description"]
    inlines = [
        ThemeCssInline,
        ThemeJsInline,
    ]

class SiteForm(ModelForm):
    class Meta:
        model = Site
        exclude = []
        widgets = {
            "published": DjangoToggleSwitchWidget(klass="django-toggle-switch-primary"),
        }

class SiteAdmin(
        DjangoReadEditSwitchAdmin,
        DjangoCardsAdminMixin,
        DjangoVisitOnSiteInNewWindowAdmin,
        admin.ModelAdmin):
    form = SiteForm
    result_cards_columns = 3
    list_display = ["name", "code", "published", "published_time", "preview_link"]
    search_fields = ["name", "code"]
    readonly_fields = ["published_time"]
    result_card_body_height = 200
    fieldsets = [
        (_("Basic Info"), {
            "fields": ["name", "code", "theme", "index_page_code"],
        }),
        (_("Publish State"), {
            "fields": ["published", "published_time"],
        }),
        (_("Other Config"), {
            "fields": ["favicon"],
        }),
    ]

    def preview_link(self, obj):
        return format_html(
            """<a href="{0}" target="_blank">{1}</a>""",
            obj.get_absolute_url(),
            _("Preview"),
        )

    preview_link.short_description = _("Preview")

    class Media:
        css = {
            "all": [
                "fontawesome/css/all.min.css",
            ]
        }

    def result_card_link_title(self, item):
        return _("Enter into Site Manager...")

    @csrf_protect_m
    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        if object_id:
            request.session[CURRENT_SIET_ID_SESSION_KEY] = object_id
        return super().changeform_view(request, object_id, form_url, extra_context)

class PageWidgetInline(DjangoReadEditSwitchAdmin, admin.TabularInline):
    model = PageWidget
    classes = ["tab-page-widgets"]

class PageAdmin(
        DjangoReadEditSwitchAdmin,
        DjangoChangelistToolbarAdminMixin,
        DjangoVisitOnSiteInNewWindowAdmin,
        DjangoTabbedChangeformAdmin,
        DraggableMPTTAdmin,
        admin.ModelAdmin):
    list_display = ["tree_actions", "display_title", "display_page_url", "template", "preview_link"]
    list_display_links = ["display_title"]

    def display_title(self, obj):
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            obj._mpttfield('level') * self.mptt_level_indent,
            obj.name,
        )
    display_title.short_description = _('Title')

    def display_page_url(self, obj):
        return "page:{0}:{1}".format(obj.site.code, obj.code)
    display_page_url.short_description = _("Display Page URL") 

    def preview_link(self, obj):
        return format_html(
            """<a href="{0}" target="_blank">{1}</a>""",
            obj.get_absolute_url(),
            _("Preview"),
        )
    preview_link.short_description = _("Preview")

    def get_queryset(self, request):
        site_id = request.session[CURRENT_SIET_ID_SESSION_KEY]
        if site_id:
            queryset = super().get_queryset(request).filter(site__id=int(site_id))
            return queryset
        else:
            return None

    fieldsets = [
        (None, {
            "fields": ["site", "parent", "name", "code"],
            "classes": ["tab-basic"],
        }),
        (None, {
            "fields": ["template", "theme"],
            "classes": ["tab-site-styles"],
        }),
        (_("Publish Status"), {
            "fields": ["published", "published_time"],
            "classes": ["tab-publish"],
        })
    ]
    inlines = [
        PageWidgetInline,
    ]

    tabs = [
        (_("Page Basic Settings"), ["tab-basic", "tab-site-style", "tab-publish"]),
        (_("Page Template & Theme Settings"), ["tab-site-styles"]),
        (_("Page Widgets Settings"), ["tab-page-widgets"]),
    ]
class WidgetLinkInline(admin.TabularInline):
    model = WidgetLink
    extra = 0

class StaticHtmlWidgetAdmin(DjangoSubclassAdmin, admin.ModelAdmin):
    list_dipslay = ["name"]
    inlines = [
        WidgetLinkInline,
    ]

class CarouselWidgetImageInline(admin.TabularInline):
    model = CarouselWidgetImage
    extra = 0

class CarouselWidgetAdmin(DjangoSubclassAdmin, admin.ModelAdmin):
    list_dipslay = ["name"]
    inlines = [
        WidgetLinkInline,
        CarouselWidgetImageInline,
    ]

class StaticListItemInline(admin.StackedInline):
    model = StaticListItem
    extra = 0
    fieldsets = [
        [None, {
            "fields": [
                ("title", "url"),
                ("target", "order"),
                ("label", "label_class"),
            ]
        }]
    ]

class StaticListWidgetAdmin(DjangoSubclassAdmin, admin.ModelAdmin):
    list_display = ["name"]
    inlines = [
        WidgetLinkInline,
        StaticListItemInline,
    ]

class TopbarBrandInline(admin.TabularInline):
    model = TopbarBrand
    extra = 0

class TopbarWidgetAdmin(DjangoSubclassAdmin, DjangoChangelistToolbarAdminMixin, admin.ModelAdmin):
    list_display = ["name"]
    inlines = [
        WidgetLinkInline,
        TopbarBrandInline,
    ]


class ArticleContentImageInline(admin.TabularInline):
    model = ArticleContentImage
    extra = 0
    fieldsets = [
        (None, {
            "fields": ["image", "image_link"],
        }),
    ]
    readonly_fields = ["image_link"]
    classes = ["tab-content-images"]

class ArticleAdmin(
        DjangoReadEditSwitchAdmin,
        DraggableMPTTAdmin,
        DjangoTabbedChangeformAdmin,
        DjangoChangelistToolbarAdminMixin,
        admin.ModelAdmin):
    list_display = ["tree_actions", "display_title", "published", "published_time"]
    list_display_links = ["display_title"]


    def display_title(self, instance):
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            instance._mpttfield('level') * self.mptt_level_indent,
            instance.title,
        )
    display_title.short_description = _('Title')

    fieldsets = [
        (None, {
            "fields": ["site", "parent", "title", "description", "label"],
            "classes": ["tab-basic"],
        }),
        (_("Article Publish Settings"), {
            "fields": ["published", "published_time"],
            "classes": ["tab-publish"]
        }),
        (None, {
            "fields": ["content", "preview_image", "show_preview_image"],
            "classes": ["tab-content"],
        }),

    ]
    inlines = [
        ArticleContentImageInline
    ]

    tabs = [
        (_("Article Basic Settings"), ["tab-basic", "tab-publish"]),
        (_("Article Content Settings"), ["tab-content"]),
        (_("Article Content Image Settigns"), ["tab-content-images"]),
    ]

class WidgetAdmin(DjangoMsmsAdmin, DjangoChangelistToolbarAdminMixin, admin.ModelAdmin):
    list_display = ["name", "type_name"]
    list_filter = ["type_name"]
    search_fields = ["name"]

    
class ArticleListWidgetAdmin(DjangoSubclassAdmin, admin.ModelAdmin):
    list_display = ["name"]
    inlines = [
        WidgetLinkInline,
    ]

class ArticleDetailWidgetAdmin(DjangoSubclassAdmin, admin.ModelAdmin):
    list_display = ["name"]
    inlines = [
        WidgetLinkInline,
    ]



admin.site.register(Template, TemplateAdmin)
admin.site.register(Site, SiteAdmin)
admin.site.register(Page, PageAdmin)
admin.site.register(Widget, WidgetAdmin)
admin.site.register(Theme, ThemeAdmin)
admin.site.register(CarouselWidget, CarouselWidgetAdmin)
admin.site.register(StaticHtmlWidget, StaticHtmlWidgetAdmin)
admin.site.register(StaticListWidget, StaticListWidgetAdmin)
admin.site.register(TopbarWidget, TopbarWidgetAdmin)
admin.site.register(Article, ArticleAdmin)
admin.site.register(ArticleListWidget, ArticleListWidgetAdmin)
admin.site.register(ArticleDetailWidget, ArticleDetailWidgetAdmin)



import djclick as click
import yaml
import pprint
from magic_import import import_from_string
from django_power_cms.models import Template


@click.group()
def register():
    """Django-Power-Site-Template register service.
    """

@register.command()
def scan():
    """Scan and list all avaiable templates.
    """
    from django.apps import apps
    counter = 0
    for app_config in apps.get_app_configs():
        name = app_config.name + ".export_templates.templates"
        templates = import_from_string(name) or []
        for template in templates:
            counter += 1
            app_label = template.get("app_label", "")
            code = template.get("code", "")
            name = template.get("name", "")
            print("{0}\t{1}\t{2}\t{3}".format(counter, app_label, code, name))

@register.command()
@click.argument("code", nargs=1, required=True)
def register(code):
    """Register or update template by template code.
    """
    found = False
    from django.apps import apps
    for app_config in apps.get_app_configs():
        name = app_config.name + ".export_templates.templates"
        templates = import_from_string(name) or []
        for template in templates:
            template_code = template.get("code", "")
            template_name = template.get("name", "")
            app_label = template.get("app_label", "")
            if code == template_code:
                found = True
                Template.register(template)
                print("Template registered: {0} {1} {2}".format(app_label, template_code, template_name))
                break
    if not found:
        print("Template with code: {0} NOT found.".format(code))


# Generated by Django 2.2.10 on 2020-03-05 02:38

from django.db import migrations, models
import django.db.models.deletion
import django_ckeditor_5.fields
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=64, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('content', django_ckeditor_5.fields.CKEditor5Field(blank=True, null=True, verbose_name='Content')),
                ('published', models.BooleanField(verbose_name='Published')),
                ('published_time', models.DateTimeField(blank=True, null=True, verbose_name='Published Time')),
                ('preview_image', models.ImageField(blank=True, null=True, upload_to='article-preview-images', verbose_name='Preview Image')),
                ('show_preview_image', models.BooleanField(default=False, verbose_name='Show Preview Image')),
                ('order', models.IntegerField(blank=True, null=True, verbose_name='Order')),
                ('priority', models.IntegerField(default=99, verbose_name='Priority')),
                ('label', models.CharField(blank=True, choices=[('Set Top', 'Set Top'), ('Important', 'Important'), ('Good News', 'Good News'), ('Warning', 'Warning')], max_length=16, null=True, verbose_name='Label')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='django_power_cms.Article', verbose_name='Article Parent')),
            ],
            options={
                'verbose_name': 'Article',
                'verbose_name_plural': 'Articles',
            },
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('code', models.CharField(max_length=64, unique=True, verbose_name='Code')),
                ('order', models.IntegerField(blank=True, null=True, verbose_name='Order')),
                ('published', models.NullBooleanField(verbose_name='Published')),
                ('published_time', models.DateTimeField(blank=True, null=True, verbose_name='Published Time')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='django_power_cms.Page', verbose_name='Parent Page')),
            ],
            options={
                'verbose_name': 'Page',
                'verbose_name_plural': 'Pages',
            },
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('app_label', models.CharField(blank=True, max_length=64, null=True, verbose_name='App Label')),
                ('app_verbose_name', models.CharField(blank=True, max_length=64, null=True, verbose_name='App Verbose Name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('preview_image', models.CharField(blank=True, max_length=128, null=True, verbose_name='Preview Image')),
                ('template', models.CharField(max_length=128, verbose_name='Template File')),
            ],
            options={
                'verbose_name': 'Template',
                'verbose_name_plural': 'Templates',
            },
        ),
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('is_default', models.BooleanField(default=False, verbose_name='Is default theme')),
            ],
            options={
                'verbose_name': 'Theme',
                'verbose_name_plural': 'Themes',
            },
        ),
        migrations.CreateModel(
            name='Widget',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_label', models.CharField(blank=True, editable=False, max_length=64, verbose_name='App Label')),
                ('model_name', models.CharField(blank=True, editable=False, max_length=64, verbose_name='Model Name')),
                ('type_name', models.CharField(blank=True, editable=False, max_length=64, verbose_name='Type Name')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('title', models.CharField(blank=True, max_length=128, null=True, verbose_name='Title')),
                ('widget_with_border', models.BooleanField(default=False, verbose_name='Widget With border')),
                ('widget_body_padding', models.CharField(blank=True, default='10px', max_length=16, null=True, verbose_name='Widget Body Pading')),
                ('widget_style', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Style')),
                ('widget_header_style', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Header Style')),
                ('widget_body_style', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Body Style')),
                ('widget_footer_style', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Footer Style')),
                ('widget_class', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Class')),
                ('widget_header_class', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Header Class')),
                ('widget_body_class', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Body Class')),
                ('widget_footer_class', models.CharField(blank=True, max_length=256, null=True, verbose_name='Widget Footer Class')),
            ],
            options={
                'verbose_name': 'Widget',
                'verbose_name_plural': 'Widgets',
            },
        ),
        migrations.CreateModel(
            name='CarouselWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('width', models.IntegerField(blank=True, null=True, verbose_name='Width')),
                ('height', models.IntegerField(blank=True, null=True, verbose_name='Height')),
            ],
            options={
                'verbose_name': 'Carousel Widget',
                'verbose_name_plural': 'Carousel Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
        migrations.CreateModel(
            name='StaticHtmlWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('html', models.TextField(blank=True, null=True, verbose_name='HTML Code')),
            ],
            options={
                'verbose_name': 'Static HTML Widget',
                'verbose_name_plural': 'Static HTML Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
        migrations.CreateModel(
            name='StaticListWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('max_display_count', models.IntegerField(blank=True, null=True, verbose_name='Max Display Count')),
                ('empty_display_message', models.TextField(blank=True, null=True, verbose_name='Empty Display Message')),
            ],
            options={
                'verbose_name': 'Static List Widget',
                'verbose_name_plural': 'Static List Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
        migrations.CreateModel(
            name='TopbarWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('brand_link', models.CharField(blank=True, max_length=128, null=True, verbose_name='Brand Link')),
                ('brand_link_target', models.CharField(blank=True, max_length=32, null=True, verbose_name='Brand Link Target')),
                ('welcome_message_for_login_user', models.CharField(blank=True, max_length=128, null=True, verbose_name='Welcome Message For Login User')),
                ('welcome_message_for_anonymous_user', models.CharField(blank=True, max_length=128, null=True, verbose_name='Welcome Message For Anonymous User')),
                ('fix_position', models.BooleanField(default=False, verbose_name='Fix Position')),
                ('show_login_or_logout_link', models.BooleanField(default=False, verbose_name='Show Login Or Logout Link')),
                ('login_link', models.CharField(blank=True, max_length=128, null=True, verbose_name='Login Link')),
                ('logout_link', models.CharField(blank=True, max_length=128, null=True, verbose_name='Logout Link')),
                ('show_change_password_link', models.BooleanField(default=False, verbose_name='Show Change Password Link')),
                ('change_password_link', models.CharField(blank=True, max_length=128, null=True, verbose_name='Change Password Link')),
            ],
            options={
                'verbose_name': 'Topbar Widget',
                'verbose_name_plural': 'Topbar Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
        migrations.CreateModel(
            name='WidgetLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.IntegerField(choices=[(1, 'Top Left'), (2, 'Top Right'), (3, 'Bottom Left'), (4, 'Bottom Right')], verbose_name='Widget Link Role')),
                ('title', models.CharField(max_length=32, verbose_name='Title')),
                ('url', models.CharField(max_length=128, verbose_name='Url')),
                ('icon', models.CharField(blank=True, max_length=64, null=True, verbose_name='Icon Class')),
                ('target', models.CharField(blank=True, max_length=16, null=True, verbose_name='Link Target')),
                ('Widget', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='links', to='django_power_cms.Widget', verbose_name='Widget')),
            ],
            options={
                'verbose_name': 'Widget Link',
                'verbose_name_plural': 'Widget Links',
            },
        ),
        migrations.CreateModel(
            name='ThemeJs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('js', models.CharField(max_length=128, verbose_name='JS File')),
                ('order', models.IntegerField(verbose_name='Order')),
                ('theme', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='js_files', to='django_power_cms.Theme', verbose_name='Theme')),
            ],
            options={
                'verbose_name': 'Theme JS',
                'verbose_name_plural': 'Theme JSes',
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='ThemeCss',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('css', models.CharField(max_length=128, verbose_name='CSS File')),
                ('order', models.IntegerField(verbose_name='Order')),
                ('theme', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='css_files', to='django_power_cms.Theme', verbose_name='Theme')),
            ],
            options={
                'verbose_name': 'Theme CSS',
                'verbose_name_plural': 'Theme CSSes',
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='TemplateSlot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=32, verbose_name='Code')),
                ('name', models.CharField(max_length=32, verbose_name='Name')),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='slots', to='django_power_cms.Template', verbose_name='Template')),
            ],
            options={
                'verbose_name': 'Template Slots',
                'unique_together': {('template', 'code')},
            },
        ),
        migrations.CreateModel(
            name='Site',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('code', models.CharField(max_length=64, unique=True, verbose_name='Code')),
                ('published', models.BooleanField(verbose_name='Published')),
                ('published_time', models.DateTimeField(blank=True, null=True, verbose_name='Published Time')),
                ('index_page_code', models.CharField(blank=True, max_length=64, null=True, verbose_name='Index Page Code')),
                ('theme', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_power_cms.Theme', verbose_name='Theme')),
            ],
            options={
                'verbose_name': 'Site',
                'verbose_name_plural': 'Sites',
            },
        ),
        migrations.CreateModel(
            name='PageWidget',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(default=0, verbose_name='Order')),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='widget_bindings', to='django_power_cms.Page', verbose_name='Page')),
                ('slot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='django_power_cms.TemplateSlot', verbose_name='Slot')),
                ('widget', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='django_power_cms.Widget', verbose_name='Widget')),
            ],
            options={
                'verbose_name': 'Page Widget Binding',
                'verbose_name_plural': 'Page Widget Bindings',
                'ordering': ['slot', 'order'],
            },
        ),
        migrations.AddField(
            model_name='page',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pages', to='django_power_cms.Site', verbose_name='Site'),
        ),
        migrations.AddField(
            model_name='page',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_power_cms.Template', verbose_name='Template'),
        ),
        migrations.AddField(
            model_name='page',
            name='theme',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_power_cms.Theme', verbose_name='Theme'),
        ),
        migrations.CreateModel(
            name='ArticleContentImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to='article-content-images', verbose_name='Image')),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='django_power_cms.Article', verbose_name='Article')),
            ],
            options={
                'verbose_name': 'Article Content Image',
                'verbose_name_plural': 'Article Content Images',
            },
        ),
        migrations.CreateModel(
            name='TopbarBrand',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(blank=True, null=True, upload_to='brand-images', verbose_name='Image')),
                ('title', models.CharField(blank=True, max_length=64, null=True, verbose_name='Title')),
                ('url', models.CharField(blank=True, max_length=128, null=True, verbose_name='URL')),
                ('order', models.IntegerField(default=0, verbose_name='Order')),
                ('topbar', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='brands', to='django_power_cms.TopbarWidget', verbose_name='Topbar')),
            ],
            options={
                'verbose_name': 'Topbar Brand',
                'verbose_name_plural': 'Topbar Brands',
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='StaticListItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=128, verbose_name='Title')),
                ('url', models.CharField(blank=True, max_length=512, null=True, verbose_name='URL')),
                ('target', models.CharField(blank=True, max_length=32, null=True, verbose_name='URL Target')),
                ('order', models.IntegerField(default=0, verbose_name='Order')),
                ('label', models.CharField(max_length=128, verbose_name='Label')),
                ('label_class', models.CharField(max_length=32, verbose_name='Label Class')),
                ('thelist', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='lists', to='django_power_cms.StaticListWidget', verbose_name='The Static List Widget')),
            ],
            options={
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='CarouselWidgetImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to='carousel-images', verbose_name='Image')),
                ('url', models.CharField(blank=True, max_length=512, null=True, verbose_name='URL')),
                ('target', models.CharField(blank=True, max_length=32, null=True, verbose_name='URL Target')),
                ('order', models.IntegerField(default=0, verbose_name='Order')),
                ('carousel', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='django_power_cms.CarouselWidget', verbose_name='Carousel')),
            ],
            options={
                'verbose_name': 'Carousel Widget Image',
                'verbose_name_plural': 'Carousel Widget Images',
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='ArticleListWidget',
            fields=[
                ('widget_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='django_power_cms.Widget')),
                ('max_display_count', models.IntegerField(default=10, verbose_name='Max Display Count')),
                ('empty_display_message', models.TextField(blank=True, null=True, verbose_name='Empty Display Message')),
                ('root', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='django_power_cms.Article', verbose_name='Root')),
            ],
            options={
                'verbose_name': 'Article List Widget',
                'verbose_name_plural': 'Article List Widgets',
            },
            bases=('django_power_cms.widget',),
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-05 09:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0002_auto_20200305_1559'),
    ]

    operations = [
        migrations.AddField(
            model_name='template',
            name='code',
            field=models.CharField(default='', max_length=32, unique=True, verbose_name='Code'),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.11 on 2020-03-09 13:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_power_cms', '0011_auto_20200309_1735'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='templateslot',
            options={'ordering': ['template', 'order'], 'verbose_name': 'Template Slots'},
        ),
        migrations.AddField(
            model_name='templateslot',
            name='order',
            field=models.IntegerField(blank=True, null=True, verbose_name='Order'),
        ),
    ]



from django_power_cms.apps import DjangoPowerCmsConfig
from django.utils.translation import ugettext_lazy as _

templates = [{
    "code": "4ef1dbc1256149a09d6ea9b23b09a7a8",
    "name": _("Right Portlet Template"),
    "app_label": "django_power_cms",
    "app_verbose_name": DjangoPowerCmsConfig.verbose_name,
    "description": _("Right Portlet Template Description."),
    "preview_image": "django-power-cms/template-preview-image/right-portlet.png",
    "template": "django-power-cms/templates/right-portlet.html",
    "slots": [{
        "code": "topbars",
        "name": _("Slot Topbars"),
        "order": 1,
    },{
        "code": "headers",
        "name": _("Slot Headers"),
        "order": 2,
    },{
        "code": "mains",
        "name": _("Slot Mains"),
        "order": 3,
    },{
        "code": "right_portlets",
        "name": _("Right Slot Portlets"),
        "order": 4,
    },{
        "code": "footers",
        "name": _("Slot Footers"),
        "order": 5,
    }]
},{
    "code": "811ec1329c7f4d9b9d874090d6094cec",
    "name": _("Left Portlet Template"),
    "app_label": "django_power_cms",
    "app_verbose_name": DjangoPowerCmsConfig.verbose_name,
    "description": _("Left Portlet Template Description."),
    "preview_image": "django-power-cms/template-preview-image/left-portlet.png",
    "template": "django-power-cms/templates/left-portlet.html",
    "slots": [{
        "code": "topbars",
        "name": _("Slot Topbars"),
        "order": 1,
    },{
        "code": "headers",
        "name": _("Slot Headers"),
        "order": 2,
    },{
        "code": "mains",
        "name": _("Slot Mains"),
        "order": 3,
    },{
        "code": "left_portlets",
        "name": _("Left Slot Portlets"),
        "order": 4,
    },{
        "code": "footers",
        "name": _("Slot Footers"),
        "order": 5,
    }]
},{
    "code": "84121a8a2a77470bb958df38711a5ea2",
    "name": _("Both Portlet Template"),
    "app_label": "django_power_cms",
    "app_verbose_name": DjangoPowerCmsConfig.verbose_name,
    "description": _("Both Portlet Template Description."),
    "preview_image": "django-power-cms/template-preview-image/both-portlet.png",
    "template": "django-power-cms/templates/both-portlet.html",
    "slots": [{
        "code": "topbars",
        "name": _("Slot Topbars"),
        "order": 1,
    },{
        "code": "headers",
        "name": _("Slot Headers"),
        "order": 2,
    },{
        "code": "left_portlets",
        "name": _("Left Slot Portlets"),
        "order": 3,
    },{
        "code": "mains",
        "name": _("Slot Mains"),
        "order": 4,
    },{
        "code": "right_portlets",
        "name": _("Right Slot Portlets"),
        "order": 5,
    },{
        "code": "footers",
        "name": _("Slot Footers"),
        "order": 6,
    }]
},{
    "code": "5aa2b1690ee84e8697d9aecbb25d17d3",
    "name": _("No Portlet Template"),
    "app_label": "django_power_cms",
    "app_verbose_name": DjangoPowerCmsConfig.verbose_name,
    "description": _("No Portlet Template Description."),
    "preview_image": "django-power-cms/template-preview-image/no-portlet.png",
    "template": "django-power-cms/templates/no-portlet.html",
    "slots": [{
        "code": "topbars",
        "name": _("Slot Topbars"),
        "order": 1,
    },{
        "code": "headers",
        "name": _("Slot Headers"),
        "order": 2,
    },{
        "code": "mains",
        "name": _("Slot Mains"),
        "order": 3,
    },{
        "code": "footers",
        "name": _("Slot Footers"),
        "order": 4,
    }]
}]
